import sys
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from .gui import *
import os
_root=os.path.dirname(__file__)
from wk import PointDict

class Event(PointDict):
    # target=None
    # data=None
    pass


def getWidgetData(widget):
    if isinstance(widget,(QLineEdit)):
        return widget.text()
    elif isinstance(widget,QComboBox):
        return widget.currentText()
    elif isinstance(widget,QCheckBox):
        return widget.isChecked()
    else:
        raise
def setWidgetData(widget,data):
    if isinstance(widget,(QLineEdit)):
        return widget.setText(data)
    elif isinstance(widget,QComboBox):
        return widget.setCurrentText(data)
    elif isinstance(widget,QCheckBox):
        return widget.setChecked(data)
    else:
        raise
def getSignalDataChanged(widget):
    if isinstance(widget,(QLineEdit)):
        return widget.textChanged
    elif isinstance(widget,QComboBox):
        return widget.currentTextChanged
    elif isinstance(widget,QCheckBox):
        return widget.stateChanged
    else:
        raise
class Field(VWidget):
    title = None
    description = None
    default = None
    convert_func = None
    onchange = None
    options=None
    range=None
    def __init__(self,name=None,title=None,description=None,default=None,convert_func=None,onchange=None,options=None,dtype=None,parent=None,**kwargs):
        if not convert_func:
            if default is not None:
                convert_func=type(default)
        convert_func = convert_func or (lambda x: x)
        self.name=name
        self.title=title
        self.description=description
        self.default=default
        self.convert_func=self.convert_func or convert_func
        self.options=options
        self.dtype=dtype
        for k,v in kwargs.items():
            setattr(self,k,v)
        self.make_widget()
        if onchange:
            self.setOnchange(onchange)
        else:
            if parent is not None and name is not None:
                def make(parent,name):
                    def func(e):
                        parent[name]=e.data
                    return func
                self.setOnchange(make(parent,name))
    def make_widget(self):
        self.inner_widget = self.make_inner_widget()
        self.widget = self.make_outter_widget()
    def make_outter_widget(self,widget=None):
        ws=[]
        if self.title:
            ws.append(QLabel(self.title))
            ws.append(VStretch())
        # if self.default:
        #     ws.append(QLabel(self.description))
        ws.append(widget or self.inner_widget)
        return VHBoxLayout()(
                *ws
            ).detach()
    def make_inner_widget(self):
        raise NotImplementedError
    def setData(self,data):
        setWidgetData(self.inner_widget,str(data))
    def getData(self):
        data=getWidgetData(self.inner_widget)
        try:
            return self.convert_func(data)
        except ValueError:
            return data


    def setOnchange(self,func):
        def make_func(target):
            def wrapper():
                e=Event(data=self.getData(), target=target)
                func(e)
            return wrapper
        getSignalDataChanged(self.inner_widget).connect(make_func(self))
class TextField(Field):
    def make_inner_widget(self):
        w=QLineEdit()
        w.setText(str(self.default) or '')
        return w
class IntField(TextField):
    convert_func = int
    def make_inner_widget(self):
        w=QLineEdit()
        w.setMinimumSize(50,20)
        w.setText(str(self.default) or '')
        return w
class FloatField(TextField):
    convert_func = float
    def make_inner_widget(self):
        w=QLineEdit()
        w.setMinimumSize(50,20)
        w.setText(str(self.default) or '')
        return w
class ListSelectField(Field):
    def make_inner_widget(self):
        w=QComboBox()
        w.addItems([str(x) for x in self.options])
        w.setCurrentText(str(self.default))
        return w
class FileChooserField(Field):
    def make_widget(self):
        lineEdit = QLineEdit()
        lineEdit.setMinimumSize(300, 20)
        lineEdit.setText(self.default or '')
        btn = QPushButton('选择文件')
        def make_func(lineEdit):
            def func():
                path = QFileDialog().getOpenFileName(caption='选取文件', directory='./')[0]
                if path:
                    lineEdit.setText(path)
            return func
        btn.clicked.connect(make_func(lineEdit))
        self.inner_widget=lineEdit
        widget = VHBoxLayout()(
            btn, lineEdit
        )
        self.widget=self.make_outter_widget(widget)

class DirChooserField(Field):
    def make_widget(self):
        lineEdit = QLineEdit()
        lineEdit.setMinimumSize(300, 20)
        lineEdit.setText(self.default or '')
        btn = QPushButton('选择文件夹')
        def make_func(lineEdit):
            def func():
                path = QFileDialog().getExistingDirectory(caption='选取文件夹', directory='./')
                if path:
                    lineEdit.setText(path)
            return func
        btn.clicked.connect(make_func(lineEdit))
        self.inner_widget=lineEdit
        widget = VHBoxLayout()(
            btn, lineEdit
        ).detach()
        self.widget=self.make_outter_widget(widget)


class BoolField(Field):
    def make_inner_widget(self):
        w=QCheckBox()
        w.setChecked(bool(self.default))
        return w



import sys
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5 import QtGui, QtCore
import os

_root = os.path.dirname(__file__)
from vtgui.gui import *
from vtgui.fields import *
from wk import PointDict
import json
import typing
import random
import logging, sys
from .utils import split_list,KillableThread






def make_layout(dic, dst, column=1):
    fields = []
    for k, v in dic.items():
        if not isinstance(v, Field):
            if isinstance(v, VirtualField):
                v.update(parent=dst)
                v.update(name=k)
                v = make_field(**v)
            else:
                v = make_field(name=k, default=v, parent=dst)
        dst[v.name] = v.default
        fields.append(v)
    ratios = [len(fields) / column] * column
    columns = split_list(fields, ratios=ratios)
    cols = []
    for column in columns:
        cols.append(VVBoxLayout()(*column))
        cols.append(VSpacing(50))
    layout = VHBoxLayout(size_average=True)(*cols).detach()
    return layout


class VirtualField(PointDict):
    def __init__(self, title=None, field_type=None, description=None, default=None, options=None):
        if options and field_type is None:
            field_type = 'union'
        super().__init__(title=title, field_type=field_type, description=description, default=default, options=options)


class SelectDir(VirtualField):
    def __init__(self, default=None, title=None, description=None):
        if default is None:
            default = ''
        super().__init__(default=default, title=title, description=description, field_type='dir')


class SelectFile(VirtualField):
    def __init__(self, default=None, title=None, description=None):
        if default is None:
            default = ''
        super().__init__(default=default, title=title, description=description, field_type='file')


def make_field(name, parent, title=None, field_type=None, description=None, default=None, options=None):
    if title is None:
        title = name
    if field_type is None:
        if default is not None:
            field_type = type(default)
        else:
            field_type = str
    if isinstance(field_type, type):
        field_type = field_type.__name__
    type_dict = {
        'union': ListSelectField,
        'bool': BoolField,
        'str': TextField,
        'int': IntField,
        'float': FloatField,
        'file': FileChooserField,
        'dir': DirChooserField,
    }
    field_type = type_dict[field_type]
    return field_type(name=name, title=title, description=description, default=default, options=options, parent=parent)






def make_app(function, args=(), columns=3,title='VTGUI'):
    if not isinstance(columns,(tuple,list)):
        columns=[columns]*len(args)
    class MainWindow(QWidget):
        app = QApplication(sys.argv)

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            # 设置窗口标题
            self.setWindowTitle(title)
            # 设置窗口大小900*600
            self.resize(1300, 700)
            self.show()
            self.init()

        def init(self):
            # 添加TextBrowser
            console=VConsole()
            self.console=console


            arg_layouts = []
            final_args=[]
            dict_arg_count=-1
            for arg in args:
                if isinstance(arg,dict):
                    dict_arg_count+=1
                    a={}
                    arg_layouts.append(make_layout(
                        arg,a,columns[dict_arg_count],
                    ))
                    arg_layouts.append(VSpacing(30))
                    final_args.append(a)
                else:
                    final_args.append(arg)
            arg_layouts=VVBoxLayout()(*arg_layouts)
            target = lambda: function(*final_args)
            layout = VVBoxLayout()(
                arg_layouts,
                VHBoxLayout()(
                    VPushButton('run', onclicked=lambda: KillableThread.start_new_thread(target=target,name='test')),
                    VPushButton('stop', onclicked=lambda: KillableThread.kill_thread_by_name('test')),
                    VStretch(),
                ),
                VHBoxLayout()(console),
            )
            self.setLayout(layout.detach())
            console.serve_as_std()

        def run(self):
            self.show()
            sys.exit(self.app.exec_())

    win = MainWindow()
    return win


if __name__ == '__main__':
    # make_app()
    pass



import json
import typing
import random
import logging, sys
import threading
import ctypes
import multiprocessing
import uuid
import time
import inspect
import math

def split_list(lis, ratios, shuffle=False):
    if not lis:
        return None
    if len(lis) < len(ratios):
        raise Exception("List is not long enough to be split.")
    if shuffle:
        random.shuffle(lis)
    s=sum(ratios)
    nums=[]
    for r in ratios:
        n=int(len(lis)*r/s+0.5)
        nums.append(n)
    total = len(lis)
    splits = []
    current_index = 0
    for i, num in enumerate(nums):
        end_point = min(current_index + num, total)
        batch = lis[current_index:end_point]
        splits.append(batch)
        current_index = end_point
    return splits

def _async_raise(tid, exctype):
    '''Raises an exception in the threads with id tid'''
    if not inspect.isclass(exctype):
        raise TypeError("Only types can be raised (not instances)")
    res = ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(tid),
                                                     ctypes.py_object(exctype))
    print('res:',res)
    if res == 0:
        raise ValueError("invalid thread id")
    elif res != 1:
        # "if it returns a number greater than one, you're in trouble,
        # and you should call it again with exc=NULL to revert the effect"
        ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(tid), None)
        raise SystemError("PyThreadState_SetAsyncExc failed")

class KillableThread(threading.Thread):
    '''A thread class that supports raising exception in the thread from
       another thread.
    '''
    _threads={}
    @classmethod
    def start_new_thread(cls,target,name):
        if cls._threads.get(name,None) is not None:
            print('thread %s is running, if you want to rerun it, you need to kill it first.'%(name))
            return
        thread=cls(target=target)
        cls._threads[name]=thread
        thread.start()
    @classmethod
    def kill_thread_by_name(cls,name):
        if name not in cls._threads.keys():
            print('Thread %s not running.'%(name))
            return
        cls._threads[name].kill()
        del cls._threads[name]

    def _get_my_tid(self):
        """determines this (self's) thread id

        CAREFUL : this function is executed in the context of the caller
        thread, to get the identity of the thread represented by this
        instance.
        """
        if not self.isAlive():
            raise threading.ThreadError("the thread is not active")

        # do we have it cached?
        if hasattr(self, "_thread_id"):
            return self._thread_id

        # no, look for it in the _active dict
        for tid, tobj in threading._active.items():
            if tobj is self:
                self._thread_id = tid
                return tid

        # TODO: in python 2.6, there's a simpler way to do : self.ident

        raise AssertionError("could not determine the thread's id")

    def kill(self, exctype=Exception):
        try:
           tid=self._get_my_tid()
           print('killing thread %s'%(tid))
           _async_raise(self._get_my_tid(), exctype)
           self.join()
        except:
            print('Thread has been killed.')




        # self.join()



import sys
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5 import QtCore,QtGui,QtWidgets
import os
_root=os.path.dirname(__file__)
from wk import PointDict
g=PointDict()


class VirtualWidget:
    pass
def setAttributes(widget,kwargs):
    for k,v in kwargs.items():
        if k.startswith('on'):
            getattr(widget,k[2:]).connect(v)
        else:
            assert k.startswith('set')
            getattr(widget,k)(v)
    return widget
def addChild(widget,child):
    # print('adding %s to %s'%(child,widget))
    ws=(QWidget,QLabel,QLayout)
    assert isinstance(widget,ws)
    if isinstance(child,(QLayout,QHBoxLayout,QVBoxLayout)):
        widget.addLayout(child)
    elif isinstance(child,(QAction)):
        widget.addAction(child)
    elif isinstance(child,(QWidget)):
        widget.addWidget(child)
    elif isinstance(child,(VirtualWidget)):
        if isinstance(child,(VStretch)):
            widget.addStretch(child.size)
        elif isinstance(child,(VSpacing)):
            widget.addSpacing(child.size)
    else:
        raise

class VWidget:
    __qtwidget__=QWidget
    def __init__(self,*args,**kwargs):
        id=kwargs.pop('id',None)
        attrs={}
        for k,v in kwargs.items():
            if k.startswith('set') or k.startswith('on'):
                attrs[k]=v
            else:
                setattr(self,k,v)
        if id:
            self.id=id
            g[id]=self
        self.widget=self.makeRealWidget(*args)
        self.children=[]
        # print(self.__qtwidget__,self.widget)
        assert self.widget is not None
        self.setAttributes(attrs)
        self.init()
    def init(self):
        pass
    def ajust(self):
        pass
    def makeRealWidget(self,*args):
        return self.__qtwidget__(*args)
    def __call__(self, *args):
        for child in args:
            if isinstance(child,(VWidget)):
                child=child.widget
            addChild(self.widget,child)
            self.children.append(child)
        self.ajust()
        return self
    def setAttributes(self,attrs):
        setAttributes(self.widget,attrs)
    def detach(self):
        return self.widget
class VBoxLayout(VWidget):
    size_average=False
    __qtwidget__ = QBoxLayout
    def ajust(self):
        if self.size_average:
            length=len(self.children)
            weight=1024//length
            for i in range(length):
                self.widget.setStretch(i,weight)

class VVBoxLayout(VBoxLayout):
    __qtwidget__ = QVBoxLayout
class VHBoxLayout(VBoxLayout):
    __qtwidget__ = QHBoxLayout
class VLabel(VWidget):
    __qtwidget__ = QLabel
class VAction(VWidget):
    __qtwidget__ = QAction
class VLineEdit(VWidget):
    __qtwidget__ = QLineEdit
class VToolBar(VWidget):
    __qtwidget__ = QToolBar
class VComboBox(VWidget):
    __qtwidget__ = QComboBox
class VPushButton(VWidget):
    __qtwidget__ = QToolButton
    def makeRealWidget(self,text):
        btn=self.__qtwidget__()
        btn.setText(text)
        return btn
class VStretch(VirtualWidget):
    def __init__(self,size=10):
        self.size=size
class VSpacing(VirtualWidget):
    def __init__(self,size=10):
        self.size=size
class VConsole(VWidget):
    __qtwidget__ = QTextEdit

    class EmittingStream(QtCore.QObject):
        textWritten = QtCore.pyqtSignal(str)  # 定义一个发送str的信号

        def write(self, text):
            self.textWritten.emit(str(text))
        def flush(self):
            pass

    def makeRealWidget(self,*args):
        w= self.__qtwidget__()
        w.setReadOnly(True)
        return w
    def serve_as_std(self):
        sys.stdout=self.stdout_stream()
        sys.stderr=self.stderr_stream()
    def stdout_stream(self):
        return self.EmittingStream(textWritten=self.write)
    def stderr_stream(self):
        return self.EmittingStream(textWritten=self.write)
    def write(self,text):
        w=self.widget
        cursor = w.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.insertText(text)
        w.setTextCursor(cursor)
        w.ensureCursorVisible()
